@@ -21,6 +21,7 @@
 #include "kmessagewidget.h"
 
 #include <QAction>
+#include <QApplication>
 #include <QEvent>
 #include <QGridLayout>
 #include <QHBoxLayout>
@@ -46,6 +47,7 @@ public:
     QToolButton *closeButton = nullptr;
     QTimeLine *timeLine = nullptr;
     QIcon icon;
+    bool ignoreShowEventDoingAnimatedShow = false;
 
     KMessageWidget::MessageType messageType;
     bool wordWrap;
@@ -53,6 +55,7 @@ public:
     QPixmap contentSnapShot;
 
     void createLayout();
+    void applyStyleSheet();
     void updateSnapShot();
     void updateLayout();
     void slotTimeLineChanged(qreal);
@@ -73,6 +76,7 @@ void KMessageWidgetPrivate::init(KMessageWidget *q_ptr)
     QObject::connect(timeLine, SIGNAL(finished()), q, SLOT(slotTimeLineFinished()));
 
     content = new QFrame(q);
+    content->setObjectName(QStringLiteral("contentWidget"));
     content->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
     wordWrap = false;
@@ -163,6 +167,59 @@ void KMessageWidgetPrivate::createLayout()
     q->updateGeometry();
 }
 
+void KMessageWidgetPrivate::applyStyleSheet()
+{
+    QColor bgBaseColor;
+
+    // We have to hardcode colors here because KWidgetsAddons is a tier 1 framework
+    // and therefore can't depend on any other KDE Frameworks
+    // The following RGB color values come from the "default" scheme in kcolorscheme.cpp
+    switch (messageType) {
+    case KMessageWidget::Positive:
+        bgBaseColor.setRgb(39, 174,  96); // Window: ForegroundPositive
+        break;
+    case KMessageWidget::Information:
+        bgBaseColor.setRgb(61, 174, 233); // Window: ForegroundActive
+        break;
+    case KMessageWidget::Warning:
+        bgBaseColor.setRgb(246, 116, 0); // Window: ForegroundNeutral
+        break;
+    case KMessageWidget::Error:
+        bgBaseColor.setRgb(218, 68, 83); // Window: ForegroundNegative
+        break;
+    }
+    const qreal bgBaseColorAlpha = 0.2;
+    bgBaseColor.setAlphaF(bgBaseColorAlpha);
+
+    const QPalette palette = QGuiApplication::palette();
+    const QColor windowColor = palette.window().color();
+    const QColor textColor = palette.text().color();
+    const QColor border = bgBaseColor;
+
+    // Generate a final background color from overlaying bgBaseColor over windowColor
+    const int newRed = (bgBaseColor.red() * bgBaseColorAlpha) + (windowColor.red() * (1 - bgBaseColorAlpha));
+    const int newGreen = (bgBaseColor.green() * bgBaseColorAlpha) + (windowColor.green() * (1 - bgBaseColorAlpha));
+    const int newBlue = (bgBaseColor.blue() * bgBaseColorAlpha) + (windowColor.blue() * (1 - bgBaseColorAlpha));
+
+    const QColor bgFinalColor = QColor(newRed, newGreen, newBlue);
+
+    content->setStyleSheet(
+        QString::fromLatin1(".QFrame {"
+                              "background-color: %1;"
+                              "border-radius: 4px;"
+                              "border: 2px solid %2;"
+                              "margin: %3px;"
+                              "}"
+                              ".QLabel { color: %4; }"
+                             )
+        .arg(bgFinalColor.name())
+        .arg(border.name())
+        // DefaultFrameWidth returns the size of the external margin + border width. We know our border is 1px, so we subtract this from the frame normal QStyle FrameWidth to get our margin
+        .arg(q->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, nullptr, q) - 1)
+        .arg(textColor.name())
+    );
+}
+
 void KMessageWidgetPrivate::updateLayout()
 {
     if (content->layout()) {
@@ -253,75 +310,10 @@ KMessageWidget::MessageType KMessageWidget::messageType() const
     return d->messageType;
 }
 
-static QColor darkShade(QColor c)
-{
-    qreal contrast = 0.7; // taken from kcolorscheme for the dark shade
-
-    qreal darkAmount;
-    if (c.lightnessF() < 0.006) { /* too dark */
-        darkAmount = 0.02 + 0.40 * contrast;
-    } else if (c.lightnessF() > 0.93) { /* too bright */
-        darkAmount = -0.06 - 0.60 * contrast;
-    } else {
-        darkAmount = (-c.lightnessF()) * (0.55 + contrast * 0.35);
-    }
-
-    qreal v = c.lightnessF() + darkAmount;
-    v = v > 0.0 ? (v < 1.0 ? v : 1.0) : 0.0;
-    c.setHsvF(c.hslHueF(), c.hslSaturationF(), v);
-    return c;
-}
-
 void KMessageWidget::setMessageType(KMessageWidget::MessageType type)
 {
     d->messageType = type;
-    QColor bg0, bg1, bg2, border, fg;
-    switch (type) {
-    case Positive:
-        bg1.setRgb(39, 174,  96); // values taken from kcolorscheme.cpp (Positive)
-        fg.setRgb(239, 240, 241);
-        break;
-    case Information:
-        bg1 = palette().highlight().color();
-        fg = palette().highlightedText().color();
-        break;
-    case Warning:
-        bg1.setRgb(246, 116, 0); // values taken from kcolorscheme.cpp (Neutral)
-        fg.setRgb(239, 240, 241);
-        break;
-    case Error:
-        bg1.setRgb(218, 68, 83); // values taken from kcolorscheme.cpp (Negative)
-        // #357210: use darker color to improve the visibility of close button.
-        bg1 = bg1.darker(110);
-        fg.setRgb(239, 240, 241);
-        break;
-    }
-
-    // Colors
-    bg0 = bg1.lighter(110);
-    bg2 = bg1.darker(110);
-    border = darkShade(bg1);
-
-    d->content->setStyleSheet(
-        QString::fromLatin1(".QFrame {"
-                              "background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,"
-                              "    stop: 0 %1,"
-                              "    stop: 0.1 %2,"
-                              "    stop: 1.0 %3);"
-                              "border-radius: 5px;"
-                              "border: 1px solid %4;"
-                              "margin: %5px;"
-                              "}"
-                              ".QLabel { color: %6; }"
-                             )
-        .arg(bg0.name())
-        .arg(bg1.name())
-        .arg(bg2.name())
-        .arg(border.name())
-        // DefaultFrameWidth returns the size of the external margin + border width. We know our border is 1px, so we subtract this from the frame normal QStyle FrameWidth to get our margin
-        .arg(style()->pixelMetric(QStyle::PM_DefaultFrameWidth, nullptr, this) - 1)
-        .arg(fg.name())
-    );
+    d->applyStyleSheet();
 }
 
 QSize KMessageWidget::sizeHint() const
@@ -340,6 +332,13 @@ bool KMessageWidget::event(QEvent *event)
 {
     if (event->type() == QEvent::Polish && !d->content->layout()) {
         d->createLayout();
+    } else if (event->type() == QEvent::PaletteChange) {
+        d->applyStyleSheet();
+    } else if (event->type() == QEvent::Show && !d->ignoreShowEventDoingAnimatedShow) {
+        if ((height() != d->content->height()) || (d->content->pos().y() != 0)) {
+            d->content->move(0, 0);
+            setFixedHeight(d->content->height());
+        }
     }
     return QFrame::event(event);
 }
@@ -428,12 +427,14 @@ void KMessageWidget::animatedShow()
         return;
     }
 
-    if (isVisible() && (d->timeLine->state() == QTimeLine::NotRunning)) {
+    if (isVisible() && (d->timeLine->state() == QTimeLine::NotRunning) && (height() == d->bestContentHeight()) && (d->content->pos().y() == 0)) {
         emit showAnimationFinished();
         return;
     }
 
-    QFrame::show();
+    d->ignoreShowEventDoingAnimatedShow = true;
+    show();
+    d->ignoreShowEventDoingAnimatedShow = false;
     setFixedHeight(0);
     int wantedHeight = d->bestContentHeight();
     d->content->setGeometry(0, -wantedHeight, width(), wantedHeight);
